@@ -43,11 +43,13 @@ parser.add_argument("--levels", help="Number of levels in the segmentation U-Net
 parser.add_argument("--dropout", help="Dropout", type=float, default=0.0)
 parser.add_argument("--norm", help="Normalization in the network (batch or instance)", type=str, default="instance")
 parser.add_argument("--activation", help="Non-linear activations in the network", type=str, default="relu")
+parser.add_argument("--classes_of_interest", help="List of indices that correspond to the classes of interest",
+                    type=str, default="0,1")
 
 # optimization parameters
 parser.add_argument("--loss", help="Specifies the loss function (and optionally, additional parameters separated by "
                                    "hashtags and colons that specify the name) used for optimization",
-                    type=str, default="dice")
+                    type=str, default="ce")
 parser.add_argument("--lr", help="Learning rate of the optimization", type=float, default=1e-3)
 parser.add_argument("--step_size", help="Number of epochs after which the learning rate should decay",
                     type=int, default=10)
@@ -60,6 +62,7 @@ parser.add_argument("--test_batch_size", help="Batch size in the testing stage",
 
 args = parser.parse_args()
 args.input_size = [int(item) for item in args.input_size.split(',')]
+args.classes_of_interest = [int(c) for c in args.classes_of_interest.split(',')]
 loss_fn = get_loss_function(args.loss)
 
 """
@@ -97,7 +100,7 @@ test_loader = DataLoader(test, batch_size=args.train_batch_size)
 """
 print('[%s] Building the network' % (datetime.datetime.now()))
 net = UNet3D(feature_maps=args.fm, levels=args.levels, dropout_enc=args.dropout, dropout_dec=args.dropout,
-             norm=args.norm, activation=args.activation)
+             norm=args.norm, activation=args.activation, coi=args.classes_of_interest)
 
 """
     Setup optimization for training
